@@ -1110,10 +1110,7 @@ Erase it? Y/N: """))
             # all nodes for this class
             for nodeid in cl.getnodeids():
                 if self.verbose:
-                    sys.stdout.write('Exporting %s - %s\r'%(classname, nodeid))
-                    sys.stdout.flush()
-                if self.verbose:
-                    sys.stdout.write('Exporting %s - %s\r'%(classname, nodeid))
+                    sys.stdout.write('\rExporting %s - %s'%(classname, nodeid))
                     sys.stdout.flush()
                 writer.writerow(cl.export_list(propnames, nodeid))
                 if export_files and hasattr(cl, 'export_files'):
@@ -1198,7 +1195,7 @@ Erase it? Y/N: """))
                     continue
 
                 if self.verbose:
-                    sys.stdout.write('Importing %s - %s\r'%(classname, n))
+                    sys.stdout.write('\rImporting %s - %s'%(classname, n))
                     sys.stdout.flush()
 
                 # do the import and figure the current highest nodeid
