@@ -131,7 +131,7 @@ class Database(rdbms_common.Database):
         try:
             self.load_dbschema()
         except psycopg.ProgrammingError, message:
-            if str(message).find('"schema" does not exist') == -1:
+            if str(message).find('schema') == -1:
                 raise
             self.rollback()
             self.init_dbschema()
