@@ -714,8 +714,7 @@ class Client:
         # spit out headers
         self.additional_headers['Content-Type'] = mime_type
         self.additional_headers['Content-Length'] = str(len(content))
-        lmt = rfc822.formatdate(lmt)
-        self.additional_headers['Last-Modified'] = lmt
+        self.additional_headers['Last-Modified'] = rfc822.formatdate(lmt)
 
         ims = None
         # see if there's an if-modified-since...
@@ -848,7 +847,13 @@ class Client:
         try:
             call(*args, **kwargs)
         except socket.error, err:
-            if err.errno not in self.IGNORE_NET_ERRORS:
+            err_errno = getattr (err, 'errno', None)
+            if err_errno is None:
+                try:
+                    err_errno = err[0]
+                except TypeError:
+                    pass
+            if err_errno not in self.IGNORE_NET_ERRORS:
                 raise
 
     def write(self, content):
