@@ -9,6 +9,9 @@ import cgi
 class HTTPException(Exception):
     pass
 
+class LoginError(HTTPException):
+    pass
+
 class Unauthorised(HTTPException):
     pass
 
@@ -49,12 +52,11 @@ class SeriousError(Exception):
     def __str__(self):
         return '''
 <html><head><title>Roundup issue tracker: An error has occurred</title>
- <meta http-equiv="Content-Type" content="text/html; charset=utf-8;">
- <link rel="stylesheet" type="text/css" href="_file/style.css">
+ <link rel="stylesheet" type="text/css" href="@@file/style.css">
 </head>
 <body class="body" marginwidth="0" marginheight="0">
  <p class="error-message">%s</p>
 </body></html>
 '''%cgi.escape(self.args[0])
 
-# vim: set filetype=python ts=4 sw=4 et si
+# vim: set filetype=python sts=4 sw=4 et si :
