@@ -465,7 +465,10 @@ class FormParser:
                     raise FormError, msg
 
             # register that we got this property
-            if value:
+            if isinstance(proptype, hyperdb.Multilink):
+                if value != []:
+                    got_props[this][propname] = 1
+            elif value is not None:
                 got_props[this][propname] = 1
 
             # get the old value
@@ -526,6 +529,19 @@ class FormParser:
                 if got.has_key(entry):
                     required.remove(entry)
 
+            # If a user doesn't have edit permission for a given property,
+            # but the property is already set in the database, we don't
+            # require a value.
+            if not (create or nodeid is None):
+                for entry in required[:]:
+                    if not self.db.security.hasPermission('Edit',
+                                                          self.client.userid,
+                                                          self.classname,
+                                                          entry):
+                        cl = self.db.classes[self.classname]
+                        if cl.get(nodeid, entry) is not None:
+                            required.remove(entry)
+            
             # any required values not present?
             if not required:
                 continue
