@@ -1324,12 +1324,14 @@ class FileClass:
         shutil.copyfile(source, dest)
 
         mime_type = None
-        if self.getprops().has_key('type'):
+        props = self.getprops()
+        if props.has_key('type'):
             mime_type = self.get(nodeid, 'type')
         if not mime_type:
             mime_type = self.default_mime_type
-        self.db.indexer.add_text((self.classname, nodeid, 'content'),
-            self.get(nodeid, 'content'), mime_type)
+        if props['content'].indexme:
+            self.db.indexer.add_text((self.classname, nodeid, 'content'),
+                self.get(nodeid, 'content'), mime_type)
 
 class Node:
     ''' A convenience wrapper for the given node
