@@ -619,57 +619,89 @@ Emails to Roundup trackers must include a Subject: line!
 
         # Matches subjects like:
         # Re: "[issue1234] title of issue [status=resolved]"
-        open, close = config['MAILGW_SUBJECT_SUFFIX_DELIMITERS']
-        delim_open = re.escape(open)
+
+        # Alias since we need a reference to the original subject for
+        # later use in error messages
+        tmpsubject = subject
+
+        sd_open, sd_close = config['MAILGW_SUBJECT_SUFFIX_DELIMITERS']
+        delim_open = re.escape(sd_open)
         if delim_open in '[(': delim_open = '\\' + delim_open
-        delim_close = re.escape(close)
+        delim_close = re.escape(sd_close)
         if delim_close in '[(': delim_close = '\\' + delim_close
-        subject_re = r'''
-        (?P<refwd>\s*\W?\s*(fw|fwd|re|aw|sv|ang)\W\s*)*\s*   # Re:
-        (?P<quote>")?                                 # Leading "
-        (%s(?P<classname>[^\d\s]+)                    # [issue..
-           (?P<nodeid>\d+)?                           # ..1234]
-         %s)?\s*
-        (?P<title>[^%s]+)?                             # issue title
-        "?                                            # Trailing "
-        (?P<argswhole>%s(?P<args>.+?)%s)?             # [prop=value]
-        '''%(delim_open, delim_close, delim_open, delim_open, delim_close)
-        subject_re = re.compile(subject_re, re.IGNORECASE|re.VERBOSE)
+
+        matches = dict.fromkeys(['refwd', 'quote', 'classname',
+                                 'nodeid', 'title', 'args',
+                                 'argswhole'])
+
+        # Look for Re: et. al. Used later on for MAILGW_SUBJECT_CONTENT_MATCH
+        re_re = r'''(?P<refwd>(\s*\W?\s*(fw|fwd|re|aw|sv|ang)\W)+)\s*'''
+        m = re.match(re_re, tmpsubject, re.IGNORECASE|re.VERBOSE)
+        if m:
+            matches.update(m.groupdict())
+            tmpsubject = tmpsubject[len(matches['refwd']):] # Consume Re:
+
+        # Look for Leading "
+        m = re.match(r'(?P<quote>\s*")', tmpsubject,
+                     re.IGNORECASE)
+        if m:
+            matches.update(m.groupdict())
+            tmpsubject = tmpsubject[len(matches['quote']):] # Consume quote
+
+        has_prefix = re.search(r'^%s(\w+)%s'%(delim_open,
+            delim_close), tmpsubject.strip())
+
+        class_re = r'%s(?P<classname>(%s))(?P<nodeid>\d+)?%s'%(delim_open,
+            "|".join(self.db.getclasses()), delim_close)
+        # Note: re.search, not re.match as there might be garbage
+        # (mailing list prefix, etc.) before the class identifier
+        m = re.search(class_re, tmpsubject, re.IGNORECASE)
+        if m:
+            matches.update(m.groupdict())
+            # Skip to the end of the class identifier, including any
+            # garbage before it.
+            
+            tmpsubject = tmpsubject[m.end():]
+
+        # if we've not found a valid classname prefix then force the
+        # scanning to handle there being a leading delimiter
+        title_re = r'(?P<title>%s[^%s]+)'%(
+            not matches['classname'] and '.' or '', delim_open)
+        m = re.match(title_re, tmpsubject.strip(), re.IGNORECASE)
+        if m:
+            matches.update(m.groupdict())
+            tmpsubject = tmpsubject[len(matches['title']):] # Consume title
+
+        args_re = r'(?P<argswhole>%s(?P<args>.+?)%s)?'%(delim_open,
+            delim_close)
+        m = re.search(args_re, tmpsubject.strip(), re.IGNORECASE|re.VERBOSE)
+        if m:
+            matches.update(m.groupdict())
 
         # figure subject line parsing modes
         pfxmode = config['MAILGW_SUBJECT_PREFIX_PARSING']
         sfxmode = config['MAILGW_SUBJECT_SUFFIX_PARSING']
 
-        # check for well-formed subject line
-        m = subject_re.match(subject)
-        if m:
-            # check for registration OTK
-            # or fallback on the default class
-            if self.db.config['EMAIL_REGISTRATION_CONFIRMATION']:
-                otk_re = re.compile('-- key (?P<otk>[a-zA-Z0-9]{32})')
-                otk = otk_re.search(m.group('title') or '')
-                if otk:
-                    self.db.confirm_registration(otk.group('otk'))
-                    subject = 'Your registration to %s is complete' % \
-                              config['TRACKER_NAME']
-                    sendto = [from_list[0][1]]
-                    self.mailer.standard_message(sendto, subject, '')
-                    return
-            # get the classname
-            if pfxmode == 'none':
-                classname = None
-            else:
-                classname = m.group('classname')
-            if classname is None:
-                if self.default_class:
-                    classname = self.default_class
-                else:
-                    classname = config['MAILGW_DEFAULT_CLASS']
-                    if not classname:
-                        # fail
-                        m = None
+        # check for registration OTK
+        # or fallback on the default class
+        if self.db.config['EMAIL_REGISTRATION_CONFIRMATION']:
+            otk_re = re.compile('-- key (?P<otk>[a-zA-Z0-9]{32})')
+            otk = otk_re.search(matches['title'] or '')
+            if otk:
+                self.db.confirm_registration(otk.group('otk'))
+                subject = 'Your registration to %s is complete' % \
+                          config['TRACKER_NAME']
+                sendto = [from_list[0][1]]
+                self.mailer.standard_message(sendto, subject, '')
+                return
 
-        if not m and pfxmode == 'strict':
+        # get the classname
+        if pfxmode == 'none':
+            classname = None
+        else:
+            classname = matches['classname']
+
+        if not classname and has_prefix and pfxmode == 'strict':
             raise MailUsageError, _("""
 The message you sent to roundup did not contain a properly formed subject
 line. The subject must contain a class name or designator to indicate the
@@ -684,46 +716,67 @@ line. The subject must contain a class name or designator to indicate the
 Subject was: '%(subject)s'
 """) % locals()
 
-        # try to get the class specified - if "loose" then fall back on the
-        # default
-        attempts = [classname]
-        if pfxmode == 'loose':
-            if self.default_class:
-                attempts.append(self.default_class)
-            else:
-                attempts.append(config['MAILGW_DEFAULT_CLASS'])
+        # try to get the class specified - if "loose" or "none" then fall
+        # back on the default
+        attempts = []
+        if classname:
+            attempts.append(classname)
+
+        if self.default_class:
+            attempts.append(self.default_class)
+        else:
+            attempts.append(config['MAILGW_DEFAULT_CLASS'])
+
+        # first valid class name wins
         cl = None
         for trycl in attempts:
             try:
-                cl = self.db.getclass(classname)
+                cl = self.db.getclass(trycl)
+                classname = trycl
                 break
             except KeyError:
                 pass
+
         if not cl:
             validname = ', '.join(self.db.getclasses())
-            raise MailUsageError, _("""
-The class name you identified in the subject line ("%(classname)s") does not exist in the
-database.
+            if classname:
+                raise MailUsageError, _("""
+The class name you identified in the subject line ("%(classname)s") does
+not exist in the database.
 
 Valid class names are: %(validname)s
 Subject was: "%(subject)s"
+""") % locals()
+            else:
+                raise MailUsageError, _("""
+You did not identify a class name in the subject line and there is no
+default set for this tracker. The subject must contain a class name or
+designator to indicate the 'topic' of the message. For example:
+    Subject: [issue] This is a new issue
+      - this will create a new issue in the tracker with the title 'This is
+        a new issue'.
+    Subject: [issue1234] This is a followup to issue 1234
+      - this will append the message's contents to the existing issue 1234
+        in the tracker.
+
+Subject was: '%(subject)s'
 """) % locals()
 
         # get the optional nodeid
         if pfxmode == 'none':
             nodeid = None
         else:
-            nodeid = m.group('nodeid')
+            nodeid = matches['nodeid']
 
         # try in-reply-to to match the message if there's no nodeid
         inreplyto = message.getheader('in-reply-to') or ''
         if nodeid is None and inreplyto:
-            l = self.db.getclass('msg').stringFind(inreplyto=inreplyto)
+            l = self.db.getclass('msg').stringFind(messageid=inreplyto)
             if l:
-                nodeid = l[0]
+                nodeid = cl.filter(None, {'messages':l})[0]
 
         # title is optional too
-        title = m.group('title')
+        title = matches['title']
         if title:
             title = title.strip()
         else:
@@ -731,14 +784,14 @@ Subject was: "%(subject)s"
 
         # strip off the quotes that dumb emailers put around the subject, like
         #      Re: "[issue1] bla blah"
-        if m.group('quote') and title.endswith('"'):
+        if matches['quote'] and title.endswith('"'):
             title = title[:-1]
 
         # but we do need either a title or a nodeid...
         if nodeid is None and not title:
             raise MailUsageError, _("""
 I cannot match your message to a node in the database - you need to either
-supply a full designator (with number, eg "[issue123]" or keep the
+supply a full designator (with number, eg "[issue123]") or keep the
 previous subject title intact so I can match that.
 
 Subject was: "%(subject)s"
@@ -752,7 +805,7 @@ Subject was: "%(subject)s"
         # additional restriction based on the matched node's creation or
         # activity.
         tmatch_mode = config['MAILGW_SUBJECT_CONTENT_MATCH']
-        if tmatch_mode != 'never' and nodeid is None and m.group('refwd'):
+        if tmatch_mode != 'never' and nodeid is None and matches['refwd']:
             l = cl.stringFind(title=title)
             limit = None
             if (tmatch_mode.startswith('creation') or
@@ -905,8 +958,8 @@ Unknown address: %(from_address)s
         # figure what the properties of this Class are
         properties = cl.getprops()
         props = {}
-        args = m.group('args')
-        argswhole = m.group('argswhole')
+        args = matches['args']
+        argswhole = matches['argswhole']
         if args:
             if sfxmode == 'none':
                 title += ' ' + argswhole
