@@ -594,7 +594,8 @@ class IssueClass:
                 if '\n' in change:
                     value = self.indentChangeNoteValue(str(value))
                     oldvalue = self.indentChangeNoteValue(str(oldvalue))
-                    change = '\nNow:\n%s\nWas:\n%s'%(value, oldvalue)
+                    change = _('\nNow:\n%(new)s\nWas:\n%(old)s') % {
+                        "new": value, "old": oldvalue}
             m.append('%s: %s'%(propname, change))
         if m:
             m.insert(0, '----------')
