@@ -412,6 +412,14 @@ class FormTestCase(unittest.TestCase):
         self.assertEqual(self.parseForm({'boolean': ' '}, 'test', nodeid),
             ({('test', nodeid): {'boolean': None}}, []))
 
+    def testRequiredBoolean(self):
+        self.assertRaises(FormError, self.parseForm, {'boolean': '',
+            ':required': 'boolean'})
+        try:
+            self.parseForm({'boolean': 'no', ':required': 'boolean'})
+        except FormError:
+            self.fail('boolean "no" raised "required missing"')
+
     #
     # Number
     #
@@ -460,6 +468,14 @@ class FormTestCase(unittest.TestCase):
         self.assertEqual(self.parseForm({'number': ' '}, 'test', nodeid),
             ({('test', nodeid): {'number': None}}, []))
 
+    def testRequiredNumber(self):
+        self.assertRaises(FormError, self.parseForm, {'number': '',
+            ':required': 'number'})
+        try:
+            self.parseForm({'number': '0', ':required': 'number'})
+        except FormError:
+            self.fail('number "no" raised "required missing"')
+
     #
     # Date
     #
