@@ -20,22 +20,14 @@
 import unittest, os, shutil, time, imp
 
 from roundup.hyperdb import DatabaseError
-from roundup import init, backends
+from roundup.backends import get_backend, have_backend
 
 from db_test_base import DBTest, ROTest, config, SchemaTest, ClassicInitTest
 
 
-# Mysql connection data
-config.MYSQL_DBHOST = 'localhost'
-config.MYSQL_DBUSER = 'rounduptest'
-config.MYSQL_DBPASSWORD = 'rounduptest'
-config.MYSQL_DBNAME = 'rounduptest'
-config.MYSQL_DATABASE = (config.MYSQL_DBHOST, config.MYSQL_DBUSER,
-    config.MYSQL_DBPASSWORD, config.MYSQL_DBNAME)
-
 class mysqlOpener:
-    if hasattr(backends, 'mysql'):
-        from roundup.backends import mysql as module
+    if have_backend('mysql'):
+        module = get_backend('mysql')
 
     def setUp(self):
         self.module.db_nuke(config)
@@ -64,13 +56,6 @@ class mysqlSchemaTest(mysqlOpener, SchemaTest):
 
 class mysqlClassicInitTest(mysqlOpener, ClassicInitTest):
     backend = 'mysql'
-    extra_config = '''
-MYSQL_DBHOST = 'localhost'
-MYSQL_DBUSER = 'rounduptest'
-MYSQL_DBPASSWORD = 'rounduptest'
-MYSQL_DBNAME = 'rounduptest'
-MYSQL_DATABASE = (MYSQL_DBHOST, MYSQL_DBUSER, MYSQL_DBPASSWORD, MYSQL_DBNAME)
-'''
     def setUp(self):
         mysqlOpener.setUp(self)
         ClassicInitTest.setUp(self)
@@ -89,16 +74,15 @@ class mysqlSessionTest(mysqlOpener, RDBMSTest):
 
 def test_suite():
     suite = unittest.TestSuite()
-    if not hasattr(backends, 'mysql'):
+    if not have_backend('mysql'):
         print "Skipping mysql tests"
         return suite
 
-    from roundup.backends import mysql
     import MySQLdb
     try:
         # Check if we can connect to the server.
         # use db_exists() to make a connection, ignore it's return value
-        mysql.db_exists(config)
+        mysqlOpener.module.db_exists(config)
     except (MySQLdb.MySQLError, DatabaseError), msg:
         print "Skipping mysql tests (%s)"%msg
     else:
